{	PICT_CDEF		}
{}
{	A simple Control definition to allow display of backdrop picts in dialogs		}
{}
{	Reason for code:  In using the dialog manager to design dialogs it is often easier to}
{		create a nice backdrop using a dedicated drawing program and save a PICT.  Unfortunately	}
{		when using a standard dialog, the controls are drawn *first*, regardless of the ordering}
{		of the item list.  The result of this is that if a PICT is used which would cover any of the	}
{		controls, there is no convenient way to keep the buttons uncovered and visible.  The solution		}
{		I have chosen to use is to create a control which does nothing but display a PICT.  This will		}
{		allow the PICT to be drawn first (if it is the last item), an all other items will cover the PICT.	}
{}
{}
{	How to use:}
{}
{		Very simple to use.}
{}
{		1)		Compile the CDEF as a code resource of type 'CDEF', and assign it some resource number (I have		}
{		arbitrarily chosen 20 as the resource number).  }
{}
{		2)		Make a CNTL resource with Min=0 and Max=32000 and ProcID = 16 * resource number of CDEF  		}
{		(320 is appropriate for the default).  If the value is set to the number of the PICT to display, nothing			}
{		further will be needed at run time.}
{}
{		3)		When designing your dialog, put the item which corresponds to the PICT control anywhere in the dialog}
{		(the PICT will be drawn without scaling or translation in the dialog box).  The control ID is the number of		}
{		the CNTL resource made in (2) above.  Note that this is not necessarily the resource number of the CNTL		}
{		resource (the CNTL resource itself identifies the resource number of the CNTL)			}
{}
{		4)		If the PICT to be used is to be set during Run Time, all that is needed is a SetCtlValue to the resource }
{		number of the pict.  In other words, the value of this 'control' is the resource number of the		}
{		PICT to be displayed.		}
{}
{	Notes:			}
{}
{		-- There is no error checking.  The programmer is responsible for making sure there is a PICT resource		}
{			corresponding to the control value			}
{}
{}
{}
{	Legalities:		}
{}
{		The code below can be used without restriction or acknowledgement.			}
{}
{	Written by:	Ramu Reddy.							}
{			email:  ramu@u.washington.edu			}
{}
{}


unit myPictCDEF;

interface

{  Think Pascal requires the entry point of a code resource to be named main  }

	function main (variation: integer; theControl: ControlHandle; msg: integer; param: longInt): longInt;

implementation

	function main (variation: integer; theControl: ControlHandle; msg: integer; param: longInt): longInt;

		var
			result: longInt;		{	the result to return		}
			thePICT: PicHandle;
	begin
		result := 0;

		case msg of

			drawCntl: 
				begin																{	Draw the control				}
					if theControl^^.contrlVis <> 0 then
						begin
							thePICT := PicHandle(GetResource('PICT', theControl^^.contrlValue));			{  get the PICT resource }
							DrawPicture(thePICT, thePICT^^.picFrame);											{ and draw it in the default location }
									{  I guess we could figure out some way to tell the control where to put the PICT instead  	}
									{	of starting in the upper left hand corner -- probably by using the control rect.		}
									{  A pretty simple modification if any one cares to do it...		}
						end;
				end;

			calcCRgns, calcCntlRgn, calcThumbRgn: 								{  In case the calling program needs to know the size }
				begin
					param := longInt(StripAddress(Ptr(param)));											{ Mask off the high byte	 before dereferencing }
					thePICT := PicHandle(GetResource('PICT', theControl^^.contrlValue));
					RectRgn(rgnHandle(param), thePICT^^.picFrame);			{  Send the PICT fram back  }
				end;


				{ No other events need to be considered.	}
			otherwise		{ initCntl, dispCntl, dragCntl, posCntl, thumbCntl, autoTrack, testCntl }
				;
		end;

		main := result;
	end;

end.